//--------------------------------------------------------------------------------//
//-                   KJOs ( K-Junior Operating System )                         -//
//                                                                               -//
//-  Copyright (C) Frdric Lambercy, K-Team S.A. 2008                           -//
//-  This library is free software; you can redistribute it and/or               -//
//-  modify it under the terms of the GNU Lesser General Public                  -//
//-  License as published by the Free Software Foundation; either                -//
//-  version 2.1 of the License, or any later version.                           -//
//-                                                                              -//
//-  This library is distributed in the hope that it will be useful,             -//
//-  but WITHOUT ANY WARRANTY; without even the implied warranty of              -//
//-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU           -//
//-  Lesser General Public License for more details.                             -//
//-                                                                              -//
//-  You should have received a copy of the GNU Lesser General Public            -//
//-  License along with this library; if not, write to the Free Software         -//
//-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   -//
//-                                                                              -//
//-                               __  __  ________                               -//
//- K-Team S.A.                  |  |/  /|__    __|___  _____  ___  ___          -//
//- Rue Galilee 9. Y-Park,       |     / __ |  | _____|/  _  \|   \/   |         -//
//- 1400 Yverdon-les-Bains       |  |  \    |  | ____|/  /_\  |        |         -//
//- Switzerland                  |__|\__\   |__|______|_/   \_|__|\/|__|         -//
//- lambercy@k-team.com   tel:+41 24 423 89 73 fax:+41 24 423 8960               -//
//-                                                                              -//
//--------------------------------------------------------------------------------//

////////////////////////////////////////////////////////////////////////////////
/*!   \file KJunior.c
      \brief The Os functions, all functions are called through an interrupt
*/
////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
//----------------------------------------------------------------//
//-                Conversion Function                           -//
char chartohex( char valuetoconvert )
{
   char convertedval;
      if( valuetoconvert >= 'A' )
      {
            convertedval = valuetoconvert-'A'+10;
      }   
      else
      {
            convertedval = valuetoconvert -'0';
      }
      return convertedval;
}

//----------------------------------------------------------------//
//-                Robot Initialisation                          -//

void KJunior_init(void)
{
 unsigned int i=0;
 // configure ports direction
 set_tris_a(INIT_TRISA);
 set_tris_b(INIT_TRISB);
 set_tris_c(INIT_TRISC);
 set_tris_d(INIT_TRISD);
 set_tris_e(INIT_TRISE);

 // set default value for each pin
 output_a(INIT_PORTA);
 output_b(INIT_PORTB);
 output_c(INIT_PORTC);
 output_d(INIT_PORTD);
 output_e(INIT_PORTE);
  
 setup_adc(adc_clock_internal);
 
 setup_oscillator(OSC_8MHZ);              // use internal osc@8 MHz, clockout
 for (i=0;i<10;i++)                       // Init the IR table values
             {
   IR_Light[i]= 0;
   IR_Proximity[i] = 0;
 }  
 PwmMotLeft = 0;                                                                // Init the Speed to 0
 PwmMotRight = 0;
 
 // Disable IR sensor emitter
 output_low(IR_GROUP1);
 output_low(IR_GROUP2);
 output_low(IR_GROUP3);
 
 // init the pin map table for the IR sensor
 IR_Pin_Table[LEFT]              = LEFT_EMITTER;
 IR_Pin_Table[FRONTLEFT]         = FRONTLEFT_EMITTER;
 IR_Pin_Table[FRONT]             = FRONT_EMITTER;
 IR_Pin_Table[FRONTRIGHT]        = FRONTRIGHT_EMITTER;
 IR_Pin_Table[RIGHT]             = RIGHT_EMITTER;
 IR_Pin_Table[REAR]              = REAR_EMITTER;
 IR_Pin_Table[GROUNDLEFT]        = GROUNDLEFT_EMITTER;
 IR_Pin_Table[GROUNDFRONTLEFT]   = GROUNDFRONTLEFT_EMITTER;
 IR_Pin_Table[GROUNDFRONTRIGHT]  = GROUNDFRONTRIGHT_EMITTER;
 IR_Pin_Table[GROUNDRIGHT]       = GROUNDRIGHT_EMITTER;
 IR_Pin_Table[6]                 = PIN_C5;                                      // Set a none use pin to the empty table
 IR_pin_Table[7]                 = PIN_C5;

 
 // Display boot message
 printf("\r\nK-Junior Operating System\r\n");
 printf("kJOS %s-%02d (%s), K-Team SA\r\n",KJ_FW_VERSION,KJ_FW_REVISION, KJ_FW_REFERENCE );
 printf("E-Mail : info@k-team.com\r\n");
 

 // Setup the adc converter to start with the LEFT IR sensor
 setup_adc(adc_clock_internal);
 setup_adc_ports(sAN0 | sAN1 | sAN2 | sAN3 | sAN4 | sAN5 | sAN8| sAN9 | sAN10 | sAN11);
 set_adc_channel( LEFT );
 
 
 // Set the timer 1 used for the internal tasks management
 set_timer1(TIMER1_VALUE);                                                      // set default value to get an interrupt each 250us
 setup_timer_1(T1_INTERNAL);                                                    // used for scheduler
 // Activate the interrupts
 enable_interrupts(INT_TIMER1);
 if( Enable_RS232_Control == 1 )
 {
    enable_interrupts(INT_RDA);
 }
 // Enable and configure the interrupts
 ext_int_edge(H_TO_L);                                                         
 enable_interrupts(INT_EXT);
 enable_interrupts(GLOBAL);
 // Read the Switch the first time
 Switchs  =   !input(PIN_A4);        
 Switchs |= ((!input(PIN_E1))<<1);   
 Switchs |= ((!input(PIN_E2))<<2);   
 
 set_timer2(0);
 setup_timer_2(T2_DIV_BY_4,172,3);                                              // used for rc5 decoder and emitter, disable when not use


}

//---------------------------------------------------------------//
//-                        TV Remote                            -//

// TV Remote Control
void TV_Remote_Control( void )
{
   switch( TV_DATA )
   {
      case 1 :
         PwmMotLeft = 5;
         PwmMotRight = 20;
         break;
      case 2 :
         PwmMotLeft = 10;
         PwmMotRight = 10;
         break;
      case 3 :
         PwmMotLeft = 20;
         PwmMotRight = 5;
         break;
      case 4:
         PwmMotLeft = -10;
         PwmMotRight = 10;
         break;
      case 5 :
         PwmMotLeft = 0;
         PwmMotRight = 0;
         break;
      case 6 :
         PwmMotLeft = 10;
         PwmMotRight = -10;
         break;
      case 7 :
         PwmMotLeft = -5;
         PwmMotRight = -20;
         break;
      case 8:
         PwmMotLeft = -10;
         PwmMotRight = -10;
         break;
      case 9 :
         PwmMotLeft = -20;
         PwmMotRight = -5;
         break;
      case 12 :            // Touche On/Off
         break;
      case 16 :            // Touche Son +
         break;
      case 17 :            // Touche Son -
         break;
   }
}


//-----------------------------------------------------------//
//                  Internal Interrupts                     -//


#INT_TIMER1
void Sheduler_Interrupt(void)                                                   // Internal Task Manager, Interrupt every 250 us
{
   unsigned int16 Timer1_temp;
   
   set_timer1(TIMER1_VALUE);                                                    // set default value to get an interrupt each 200us

//--------------------------------------------------------------------------------//
//- Time Function                                                                -//
//--------------------------------------------------------------------------------//  
   if(++TimeTipDivider >= 4)
   {
      TimeTipDivider = 0;
      TimeTip++;                                                                // TimeTip incremented every 1 ms (counter up to 4294967296 ms = 1193 hours ...)
      if(MotorTime > 0)                                                         // If the motor must be stopped after n ms
      {
        if(++MotorTimeDivider >= 100)                                           // each unit is 10ms
        {
          MotorTimeDivider = 0;
          if(--MotorTime == 0)                                                  // If the time reach its end
          {
           PwmMotRight = 0;                                                     // Stop the motor
           PwmMotLeft = 0;
          }
        
        }
      
      }

//--------------------------------------------------------------------------------//
//- Read the value of the mode switches all the ms                               -//
//--------------------------------------------------------------------------------//
      Switchs  =   !input(PIN_A4);        // MODE1
      Switchs |= ((!input(PIN_E1))<<1);   // MODE2
      Switchs |= ((!input(PIN_E2))<<2);   // MODE3
   }
//--------------------------------------------------------------------------------//
//- Motor PWM generator 250Hz                                                    -//
//--------------------------------------------------------------------------------//    
                                                                                // Each PWM can be configured from -20 to 20, then to obtain a PWM frequency of 250Hz

   if(PwmMotLeft == 0)                                                          // Turn off the motor left if the PWM is 0
   {
      output_low(MotG1);
      output_low(MotG2);
   }
   else if(PwmMotLeft > 0)                                                      // Speed positive
   {
      if(PwmMotLeft <= PWMCounter)                                              // The ratio is reach, clear the output 
      {
         output_low(MotG1);
         output_low(MotG2);      
      }
      else                                                                      // The ratio is not reach, turn on the H bridge
      {
         output_high(MotG1);
         output_low(MotG2);      
      }
   }
   else if(PwmMotLeft < 0)                                                      // Speed positive
   {
      if(-PwmMotLeft <= PWMCounter)                                              // The ratio is reach, clear the output 
      {
         output_low(MotG1);
         output_low(MotG2);      
      }
      else                                                                      // The ratio is not reach, turn on the H bridge
      {
         output_low(MotG1);
         output_high(MotG2);      
      }
   }   
   if(PwmMotRight == 0)                                                         // Turn off the motor left if the PWM is 0
   {
      output_low(MotD1);
      output_low(MotD2);
   }
   else if(PwmMotRight > 0)                                                     // Speed positive
   {
      if(PwmMotRight <= PWMCounter)                                             // The ratio is reach, clear the output 
      {
         output_low(MotD1);
         output_low(MotD2);      
      }
      else                                                                      // The ratio is not reach, turn on the H bridge
      {
         output_high(MotD1);
         output_low(MotD2);      
      }
   }   
   else if(PwmMotRight < 0)                                                     // Speed positive
   {
      if(-PwmMotRight <= PWMCounter)                                             // The ratio is reach, clear the output 
      {
         output_low(MotD1);
         output_low(MotD2);      
      }
      else                                                                      // The ratio is not reach, turn on the H bridge
      {
         output_low(MotD1);
         output_high(MotD2);      
      }
   }      
   if(++PWMCounter >=20)                                                        // Increment and clear the counter if 20 is reached
   {
      PWMCounter = 0;
   }

//--------------------------------------------------------------------------------//
//- Buzzer PWM Generator                                                         -//
//--------------------------------------------------------------------------------//  
   if(buzzValue == 0)                                                           // Turn OFF the buzzer if no frequency is set
      output_low(BUZZER);
   else
   {
      if(++buzzCounter >= (100 - buzzValue))                                    // If the counter reach the value
      {
         output_bit(BUZZER, !input(BUZZER));                                    // toggle the Buzzer input
         buzzCounter = 0;
      }    
   
   }   

//--------------------------------------------------------------------------------//
//- manage the Auto mode of the LED, each LED is ON during 100ms                 -//
//--------------------------------------------------------------------------------//
   if(LED_mode != 0)                                                            // If one of the LED is in auto mode
   {
      switch(LED_Counter)
      {
         case 0:
            if(LED_mode & 0x10)                                                 // Check if the LED4 is in auto mode
               output_low(LED4);                                                // Turn OFF the LED4         
            if(LED_mode & 0x01)                                                 // Check if the LED0 is in auto mode
               output_high(LED0);                                               // Turn ON the LED0
            break;
         case 500:
            if(LED_mode & 0x01)                                                 // Check if the LED0 is in auto mode
               output_low(LED0);                                                // Turn OFF the LED0
            if(LED_mode &0x02)                                                  // Check if the LED1 is in auto mode
               output_high(LED1);                                               // then turn ON the LED1
            break;
         case 1000:
            if(LED_mode & 0x02)                                                 // Check if the LED1 is in auto mode
               output_low(LED1);                                                // Turn OFF the LED1
            if(LED_mode &0x04)                                                  // Check if the LED2 is in auto mode
               output_high(LED2);                                               // then turn ON the LED2
            break;         
         case 1500:
            if(LED_mode & 0x04)                                                 // Check if the LED2is in auto mode
               output_low(LED2);                                                // Turn OFF the LED2
            if(LED_mode &0x08)                                                  // Check if the LED3is in auto mode
               output_high(LED3);                                               // then turn ON the LED3
            break;    
         case 2000:
            if(LED_mode & 0x08)                                                 // Check if the LED2is in auto mode
               output_low(LED3);                                                // Turn OFF the LED2
            if(LED_mode &0x10)                                                  // Check if the LED3is in auto mode
               output_high(LED4);                                               // then turn ON the LED3
            break;             
      }
      if(++LED_Counter >= 2499)
         LED_Counter = 0;
   
   }

//--------------------------------------------------------------------------------//
//- IR Sensor Management  (10ms to refresh all the IR sensor)                    -//
//--------------------------------------------------------------------------------//     
   if( (Auto_Refresh_Sensors == 1) || (Manual_Refresh_sensors == 1) )
   {
      // Read ambiant value
      if(IR_ADC_Flag == 1)
      {
         IR_Light[ IR_counter ] = read_adc(ADC_READ_ONLY);                      // Read the ambiant value 
         output_high(IR_Pin_Table[ IR_Counter ]);                               // Activate the IR emitter
         IR_ADC_Flag++;
      }
      // Read proximity value
      else if(IR_ADC_Flag == 3)
      {
         IR_Proximity[ IR_Counter ] = read_adc(ADC_READ_ONLY);                  // read the analog value of the proximity
         IR_Proximity[ IR_Counter ] = abs(IR_light[IR_Counter] - IR_Proximity[ IR_Counter ]); // Substract the proximity to the ambiant value to obtain the correct distance value
         output_low(IR_Pin_Table[ IR_Counter ]);                                   // Disable the IR emitter
         if(++IR_Counter > GROUNDRIGHT)                                            // Select another sensor
         {
            IR_Counter = 0;
            Sensors_Refreshed_Flag = 1;                                            // Set the flag to indicate that all the sensor was refresh
            Manual_Refresh_sensors = 0;                                            // reset the manual refresh flag, to refresh only one time
         }
         set_adc_channel( IR_Counter );                                            // Change the adc input channel
         IR_ADC_Flag = 0;                                                          // Set the flag to phase 2 (waiting phase)         
      }
      else
      {
         read_adc(ADC_START_ONLY);                                              // Start an ADC conversion to read the IR SENSOR
         IR_ADC_Flag++;                                                         
      }

   }

   // Protection if the interrupt is too long
   Timer1_temp = get_timer1();                                                  // Verify if the Timer1 has roll-on or not
   if(Timer1_temp < TIMER1_VALUE)                                               // If the timer1 value is smaller than the previous consigne
      set_timer1(65500);                                                        // reset the timer1 to have an interrupt after a small time

}


#INT_EXT
void TV_Remote_Interrupt(void)
{
   if( Auto_Refresh_TV_Remote == 1 )                                            // Only if the Auto refresh TV flag is set
   {
       TV_Counter=0;                                                            // Reset the TV Counter
       TV_table[TV_Counter] = !input(IR_IN);                                    // read the first start bit, invert it to match with the modulation of the receiver
       disable_interrupts(INT_EXT);
       setup_timer_2(T2_DIV_BY_4,190,3);                                        // The first time is shorter to read on the beginning of the coding
       set_timer2(0);
       enable_interrupts(INT_TIMER2);  
       TV_Data_pending = 1;                                                     // Indicate that a reception of a TV Data is in progress
   }
   
}

//--------------------------------------------------------------------------------//
//- Modulate at 36 kHz with 25 duty-cycle                                        -//
//--------------------------------------------------------------------------------//
void KJunior_IR_Send32x(void)
{
 unsigned int i = 0;
 
 for (i=0;i<32;i++)
  {
   output_bit(IR_OUT,1);
   delay_us(13);                                                                 // This value was choose to match exactly with the rc5 timing
   output_bit(IR_OUT,0);
   delay_us(8);                                                                 // This value was choose to match exactly with the rc5 timing
  }
}


#INT_TIMER2
void RC5_Decoding_Interrupt(void)
{
 
   int1 RC5_Bit = 0;
   
   if(TV_Data_pending == 1)                                                     // Verify if a data is coming on the receiver, or ...
   {
      set_timer2(0);
      setup_timer_2(T2_DIV_BY_4,172,5);                                         // Interrupt every 1.780 ms
      if(++TV_Counter <= 13)
      {
         TV_Table[TV_Counter] = input(IR_IN);                                   // Read the bit of the manchester coding
      }
      else
      {
         if(TV_Table[StartBit1] == 1 && TV_Table[StartBit2] == 1)               // Verify that the two start bit are correct (value must be 1)
         {
            // If correct, save the data and address of the receiveing command
            TV_ADDR = (TV_Table[AddrBit4] << 4) |(TV_Table[AddrBit3] << 3) | (TV_Table[AddrBit2] << 2) | (TV_Table[AddrBit1] << 1) | (TV_Table[AddrBit0]);
            TV_DATA = (TV_Table[DataBit5] << 5) | (TV_Table[DataBit4] << 4) | (TV_Table[DataBit3] << 3) | (TV_Table[DataBit2] << 2) | (TV_Table[DataBit1] << 1) | (TV_Table[DataBit0]);
            TV_DATA_Available = 1;                                              // Set the flag to know that a TV data is available
            if(Enable_TV_Remote_Control == 1)                                   // If the control remote is activate
               TV_Remote_Control();                                             // Set the speed for each motor
         }
         enable_interrupts(INT_EXT);                                            // Enable the external interrupt after receiving a complete data
         disable_interrupts(INT_TIMER2);                                        // Disable the timer 2 interrupt
         setup_timer_2(T2_DISABLED,172,4);                                      // stop the timer2
         bit_clear(INTCON,1);                                                   // Clear the INTF bit to avoid a interrupt on the RB port
         TV_Data_pending = 0;                                                   // Clear the flag to indicate that the data is completely received
      }
   }
   else if(TV_Data_emitting == 1)                                               // .. if a data must be send on the emitter
   {
      setup_timer_2(T2_DIV_BY_4,200,2);                                         // Interrupt every 886 us   
      do
      {
         TV_Data_loop = 0;                                                         // reset the flag, the loop can be done only one times
         set_timer2(0);
         if(TV_Counter < 3)                                                        // the first 3 bits are always 1 (start bit and toggle)
            RC5_Bit = 1;
         else if(TV_Counter > 13)                                                  // if all the bits are send, 
         {
            TV_Data_emitting = 0;                                                  // Stop the emitting
            enable_interrupts(INT_EXT);                                            // Enable the external interrupt after emitting a complete rc5 code
            disable_interrupts(INT_TIMER2);                                        // Disable the timer 2 interrupt
            setup_timer_2(T2_DISABLED,215,2);                                      // stop the timer2
            setup_timer_1(T1_INTERNAL);                                            // Enable the Timer1
            enable_interrupts(INT_TIMER1);
            bit_clear(INTCON,1);                                                   // Clear the INTF bit to avoid a interrupt on the RB port
            TV_Counter_flag = 0;                                                   // Set the flag and the rc5 bit to ensure the output will be down
            RC5_Bit = 1;                                                          
         }
         else if((TV_Counter > 8) && (TV_Counter <= 13))                           // The last 6 bits are the Data
         {
            if(TV_Data_send & (0x20 >> (TV_Counter - 8)))                          // Mask the Data to keep only the bit to send
               RC5_Bit = 1 ;                 
            else 
               RC5_Bit = 0;         
         }
         else                                                                      // Address Bits
         {
            if(TV_Addr_send & (0x10 >> (TV_Counter - 3)))                          // Mask the Data to keep only the bit to send
               RC5_Bit = 1 ;                 
            else 
               RC5_Bit = 0;         
         }      
         if((TV_Counter_flag == 0) && (RC5_bit == 0))                              // First phase of bit = 0
         {
//            disable_interrupts(INT_TIMER1);                                     // Disable the timer1 interrupt to ensure a proper timing
            KJunior_IR_Send32x();                                               // Modulate the bit at 36kHz to send on the emitter
//            enable_interrupts(INT_TIMER1);                                      // Enable timer 1 interrupt
            TV_Counter_flag = 0;                                                   // Clear the flag to go to the next bit,  
            TV_Counter++;                                                          // No need to make a second phase
            set_timer2(0);                                                         // reset the timer2 to have a proper time
            bit_clear(PIR1,1);                                                     // clear the flag of the Timer2 interrupt
         }
         else if((TV_Counter_flag == 1) && (RC5_bit == 1))                         // Second phase when a bit is = 1
         {
//            disable_interrupts(INT_TIMER1);                                     // Disable the timer1 interrupt to ensure a proper timing
            KJunior_IR_Send32x();                                               // Modulate the bit at 36kHz to send on the emitter
//            enable_interrupts(INT_TIMER1);                                      // Enable timer 1 interrupt            TV_Counter_flag = 0;                                                   // Reset the flag to go the first phase of the next bit
            TV_Counter++;                                                          // Increment the Counter to send the next bit
            TV_Counter_flag = 0; 
            TV_Data_loop = 1;
            set_timer2(0);                                                         // reset the timer2 to have a proper time
            bit_clear(PIR1,1);                                                     // clear the flag of the Timer2 interrupt         
         }
         else
         {
            TV_Counter += TV_Counter_Flag;                                         // If the flag is set (second phase) increment the counter
            TV_Counter_flag = !TV_Counter_flag;                                    // Change phase of bit
            output_low(IR_OUT);                                                    // Clear the IR output
         
         }
      
      }while(TV_Data_loop == 1);                                                   // loop to manage the case when a 1 is following by a zero (emitting two times with no interruption)
   }
   else                                                                         // Otherwise 
   {
      enable_interrupts(INT_EXT);                                               // Enable the external interrupt 
      disable_interrupts(INT_TIMER2);                                           // Disable the timer 2 interrupt
      setup_timer_2(T2_DISABLED,172,4);                                         // stop the timer2
      bit_clear(INTCON,1);                                                      // Clear the INTF bit to avoid a interrupt on the RB port

   }
   
}

#separate
void __default()
{
   printf( "Unknown Command ?!\r\n" );
}

//--------------------------------------------------------------------------------//
//- Reset the KJunior uC                                                         -//
//--------------------------------------------------------------------------------//  
#separate
void __Z()
{
   printf("z\r\n");
   reset_cpu();
}

//--------------------------------------------------------------------------------//
//- Set the speed of the 2 motors during n second. Value are +/- 20 for the speed //
//  and 0-255 for the time (unit base is 
//--------------------------------------------------------------------------------//
#separate
void __A(void)
{
   unsigned char Arg[5];
   unsigned char i, j;
   signed int8 MotG_temp;
   i = 2;                                                                       // Begin with the 3 char (after the first ",")
   j = 0;
   // While there no END of line "\n" or End of Char ",", place the value in a table to convert it 
   while((SerialBuffer[ i ] != ',') &&  (SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
   {
      Arg[j] = SerialBuffer[ i ];
      i++;     
      j++;
   }
   Arg[j] ='\0';                                                                // PLace a End of sentence at the end of the values
   MotG_Temp = atoi(Arg);                                                       // Convert the table of value into a signed int8
   if((SerialBuffer[ i ] == 10) || (SerialBuffer[ i ] == 13))                   // If the second argument is not set (i.e. "D,20\r\n") return a error
   {
      printf("Unknown Command ?!\r\n" );
   }
   else
   {
      i++;
      j = 0;
      // While there no END of line "\n" place the value in a table to convert it 
      while((SerialBuffer[ i ] != ',') &&  (SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
      {
         Arg[j] = SerialBuffer[ i ];
         i++;     
         j++;
      }
      Arg[j] ='\0';                                                             // PLace a End of sentence at the end of the values
      if((SerialBuffer[ i ] == 10) || (SerialBuffer[ i ] == 13))                   // If the second argument is not set (i.e. "D,20\r\n") return a error
      {
        printf("Unknown Command ?!\r\n" );
      }
      else
      {
        PwmMotLeft = MotG_Temp;
        PwmMotRight = atoi(Arg);  
        i++;
        j = 0;
        // While there no END of line "\n" place the value in a table to convert it 
        while((SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
        {
           Arg[j] = SerialBuffer[ i ];
           i++;     
           j++;
        }
        Arg[j] ='\0';                                                             // PLace a End of sentence at the end of the values        
        if(PwmMotLeft >20)                                                        // Limit the minimum and maximum PWM value to -20 and 20
           PwmMotLeft = 20;
        else if(PwmMotLeft < -20) 
           PwmMotLeft = -20;
        if(PwmMotRight >20)                                                       // Limit the minimum and maximum PWM value to -20 and 20
          PwmMotRight = 20;
        else if(PwmMotRight < -20) 
          PwmMotRight = -20;           
        MotorTime = atoi(Arg);                                                    // Convert the table of value into a unsigned char               
        printf("a\r\n"); 
      }
      
   }
}

//--------------------------------------------------------------------------------//
//- Read the KJunior OS version and revision                                     -//
//--------------------------------------------------------------------------------//  
#separate
void __B(void)
{
   printf( "b,%s-%02d\r\n" , KJ_FW_VERSION,KJ_FW_REVISION );
}

//--------------------------------------------------------------------------------//
//- Set the speed of the 2 motors. Valure are +/- 20 for each one                -//
//--------------------------------------------------------------------------------//
#separate
void __D(void)
{
   unsigned char Arg[5];
   unsigned char i, j;
   signed int8 MotG_temp;
   i = 2;                                                                       // Begin with the 3 char (after the first ",")
   j = 0;
   // While there no END of line "\n" or End of Char ",", place the value in a table to convert it 
   while((SerialBuffer[ i ] != ',') &&  (SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
   {
      Arg[j] = SerialBuffer[ i ];
      i++;     
      j++;
   }
   Arg[j] ='\0';                                                                // PLace a End of sentence at the end of the values
   MotG_Temp = atoi(Arg);                                                       // Convert the table of value into a signed int8
   if((SerialBuffer[ i ] == 10) || (SerialBuffer[ i ] == 13))                   // If the second argument is not set (i.e. "D,20\r\n") return a error
   {
      printf("Unknown Command ?!\r\n" );
   }
   else
   {
      i++;
      j = 0;
      // While there no END of line "\n" place the value in a table to convert it 
      while((SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
      {
         Arg[j] = SerialBuffer[ i ];
         i++;     
         j++;
      }
      Arg[j] ='\0';                                                             // PLace a End of sentence at the end of the values
      PwmMotLeft = MotG_Temp;
      if(PwmMotLeft >20)                                                        // Limit the minimum and maximum PWM value to -20 and 20
         PwmMotLeft = 20;
      else if(PwmMotLeft < -20) 
         PwmMotLeft = -20;
      PwmMotRight = atoi(Arg);                                                  // Convert the table of value into a signed int8               
      if(PwmMotRight >20)                                                       // Limit the minimum and maximum PWM value to -20 and 20
         PwmMotRight = 20;
      else if(PwmMotRight < -20) 
         PwmMotRight = -20;
      printf("d\r\n");     
      
   }
}
//--------------------------------------------------------------------------------//
//- Read the Speed of the two motors (-20 to +20)                                -//
//--------------------------------------------------------------------------------//       
#separate
void __E()
{
   printf("e,%d,%d\r\n",PwmMotLeft,PwmMotRight);
}
//--------------------------------------------------------------------------------//
//- Set the frequency of the buzzer, 0 = Off , 1 = 500Hz, 100 = 5kHz             -//
//--------------------------------------------------------------------------------//              
#separate
void __H(void)
{
   unsigned char Arg[5];
   unsigned char i, j;
   i = 2;                                                                       // Begin with the 3 char (after the first ",")
   j = 0;
   // While there no END of line "\n" or End of Char ",", place the value in a table to convert it 
   while((SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
   {
      Arg[j] = SerialBuffer[ i ];
      i++;     
      j++;
   }
   Arg[j] ='\0';                                                                // PLace a End of sentence at the end of the values
   buzzValue = atoi(Arg);                                                       // Convert the table of value into a unsigned int8
   if(buzzvalue > 100)                                                          // Limit the value to 100
      buzzValue = 100;
   printf("h\r\n");                                                             // Send the response

}
//--------------------------------------------------------------------------------//
//- Scan the I2C Bus                                                             -//
//--------------------------------------------------------------------------------//  
#separate
void __J()
{
   unsigned char i;
   unsigned char TempVal;
   printf("j");
   for(i=3;i<254;i=i+2)                                                         // Try all the available I2C address (0x00 to 0xFF)
   {
      i2c_start();
      if(i2c_write(i-1)==0)                                                     // If a device is detected
      {
         i2c_write(0x00);                                                       // Version Register
         i2c_start();
         i2c_write(i);
         TempVal=i2c_read(0);                                                   // Read the Version register to validate the communication
         i2c_stop();
         printf(",%2x",i-1);
      }
      else
      {
         i2c_stop();                                                            // Stop the I2C communication
      }
   }
   printf("\r\n");
}
//--------------------------------------------------------------------------------//
//- Set the LED value                                                            -//
//--------------------------------------------------------------------------------//  
#separate
void __L(void)
{
   unsigned char LED_temp;

  switch(SerialBuffer[2])
   {
      case '0':
         LED_temp = LED0;
         break;
      case '1':
         LED_temp = LED1;
         break;
      case '2':
         LED_temp = LED2;
         break;
      case '3':
         LED_temp = LED3;
         break;
      case '4':
         LED_temp = LED4;
         break;
      default:
         printf("Unknown Command ?!\r\n" );
         break;
   }
   if(SerialBuffer[4] == '3')                                                   // Activate the auto mode of the LED
      LED_Mode |= (0x01 << (SerialBuffer[2] - '0'));
   else
   {
      if(SerialBuffer[4] == '2')                                                // Toggle the Selected LED
      {
         if(input(LED_temp))
            output_low(LED_TEMP);
         else
            output_high(LED_TEMP);
      }
      else if(SerialBuffer[4] == '0')
          output_low(LED_temp);                                                 // Turn OFF the selected LED
      else                                                  
         output_high(LED_temp);                                                 // Turn ON the selected LED
      LED_Mode &= 0xff - (0x01 << (SerialBuffer[2] - '0'));                     // Reset the flag of the auto mode
   }               
   printf("l\r\n");
}


//--------------------------------------------------------------------------------//
//- Return the values of the IR Proximity sensor                                 -//
//--------------------------------------------------------------------------------//   
#separate
void __N(void)
{
   printf( "n,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld\r\n" , IR_Proximity[ LEFT ] , IR_Proximity[ FrontLEFT ] ,
         IR_Proximity[ FRONT ] , IR_Proximity[ FRONTRIGHT] , IR_Proximity[ RIGHT] , IR_Proximity[ REAR ] ,
         IR_Proximity[ GROUNDLEFT ] , IR_Proximity[ GROUNDFRONTLEFT ], IR_Proximity[ GROUNDFRONTRIGHT ] , IR_Proximity[ GROUNDRIGHT ]  );
}
//--------------------------------------------------------------------------------//
//- Return the values of the IR Ambient light sensor                             -//
//--------------------------------------------------------------------------------//                  
#separate
void __O(void)
{
   printf( "o,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld,%04ld\r\n" , IR_Light[ LEFT ] , IR_Light[ FrontLEFT ] ,
         IR_Light[ FRONT ] , IR_Light[ FRONTRIGHT] , IR_Light[ RIGHT] , IR_Light[ REAR ] ,
         IR_Light[ GROUNDLEFT ] , IR_Light[ GROUNDFRONTLEFT ], IR_Light[ GROUNDFRONTRIGHT ] , IR_Light[ GROUNDRIGHT ]  );
}

//--------------------------------------------------------------------------------//
//- Read on the I2C Bus                                                          -//
//--------------------------------------------------------------------------------// 
#separate
void __R(void)
{
   unsigned char memaddress = 0;
   unsigned char rcvalue = 0;
   unsigned char nb_bytes = 0;
   unsigned char TempVal;
   unsigned char i = 0;
   memaddress = chartohex(SerialBuffer[2]);
   memaddress = (memaddress<<4) + chartohex(SerialBuffer[3]);
   rcvalue = chartohex(SerialBuffer[5]);
   rcvalue = (rcvalue<<4) + chartohex(SerialBuffer[6]);
   nb_bytes = chartohex(SerialBuffer[8]);
   nb_bytes = (nb_bytes<<4) + chartohex(SerialBuffer[9]);
   i2c_start();
   i2c_write( memaddress );                                      // Writing Address of the module
   i2c_write( rcvalue );                                         // Writing Address of the register
   i2c_start();
   i2c_write( memaddress + 1 );                                  // Writing Address of the module in Read Mode
   printf("r");
   while(nb_bytes> 1)
   {
     TempVal = i2c_read();
     printf(",%03u",TempVal);
     nb_bytes--;
   }
   TempVal = i2c_read(0);
   printf(",%03u",TempVal);
   i2c_stop();
 
   
   printf("\r\n");

}
//--------------------------------------------------------------------------------//
//- Return the receiving TV data                                                 -//
//--------------------------------------------------------------------------------// 
#separate
void __T()
{
   printf( "t,%03u,%03u\r\n" , TV_ADDR,TV_DATA );
}
//--------------------------------------------------------------------------------//
//- Send a RC5 data on the IR emitter                                            -//
//--------------------------------------------------------------------------------// 
#separate
void __V()
{
   unsigned char Arg[5];
   unsigned char i, j;
   i = 2;                                                                       // Begin with the 3 char (after the first ",")
   j = 0;
   // While there no END of line "\n" or End of Char ",", place the value in a table to convert it 
   while((SerialBuffer[ i ] != ',') &&  (SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
   {
      Arg[j] = SerialBuffer[ i ];
      i++;     
      j++;
   }
   Arg[j] ='\0';                                                                // PLace a End of sentence at the end of the values
   TV_Addr_send = atoi(Arg);                                                    // Convert the table of value into a unsigned int8
   if((SerialBuffer[ i ] == 10) || (SerialBuffer[ i ] == 13))                   // If the second argument is not set (i.e. "D,20\r\n") return a error
   {
      printf("Unknown Command ?!\r\n" );
   }
   else
   {
      i++;
      j = 0;
      // While there no END of line "\n" place the value in a table to convert it 
      while((SerialBuffer[ i ] != 10) && (SerialBuffer[ i ] != 13))
      {
         Arg[j] = SerialBuffer[ i ];
         i++;     
         j++;
      }
      Arg[j] ='\0';                                                             // PLace a End of sentence at the end of the values
      TV_Data_send = atoi(Arg);                                                 // Convert the table of value into a unsigned int8
      TV_Data_emitting = 1;                                                     // Set the flag to indicate that a mesasge must be send
      printf("v\r\n");
      while(TV_Data_pending == 1);                                              // If a IR data is receiving, wait until it's done
      set_timer2(0);
      setup_timer_2(T2_DIV_BY_4,215,2);                                         // Interrupt every 886 us
      TV_Counter = 0;                                                           // reset TV_Counter 
      TV_Counter_flag = 0;                                                      // reset the sending phase flag
      disable_interrupts(INT_EXT);                                              // Disable interrupt of the receiving IR, to avoid receiving its own data
      disable_interrupts(INT_TIMER1);                                           // disable the timer1 interrupt to avoid perturbation
      enable_interrupts(INT_TIMER2);                                            // enable the timer2 interrupt to start the emitting
   }      
      
}

//--------------------------------------------------------------------------------//
//- Write a data on the I2C Bus                                                  -//
//--------------------------------------------------------------------------------// 
#separate
void __W(void)
{
   unsigned char rcvalue = 0;
   rcvalue = chartohex(SerialBuffer[2]);
   rcvalue = (rcvalue<<4) + chartohex(SerialBuffer[3]);
   i2c_start();
   i2c_write( rcvalue );                                         // Writing Address of the module
   rcvalue = chartohex(SerialBuffer[5]);
   rcvalue = (rcvalue<<4) + chartohex(SerialBuffer[6]);
   i2c_write( rcvalue );                                         // Writing Address of the register
   rcvalue = chartohex(SerialBuffer[8]);
   rcvalue = (rcvalue<<4) + chartohex(SerialBuffer[9]);
   i2c_write( rcvalue );                                         // Writing Data in the register
   i2c_stop();
   printf("w\r\n");

}

void __X(void)
{
            printf("x%c%c%c%c",IR_Proximity[ LEFT ] & 0x00FF, (IR_Proximity[ LEFT ]>>8) & 0x00FF  , IR_Proximity[ FrontLEFT ]  & 0xFF , (IR_Proximity[ FrontLEFT ] >> 8)  & 0xFF);
            printf("%c%c%c%c",IR_Proximity[ FRONT ]  & 0xFF, (IR_Proximity[ FRONT ] >> 8) & 0xFF , IR_Proximity[ FRONTRIGHT] & 0xFF, (IR_Proximity[ FRONTRIGHT] >> 8) & 0xFF );
            printf("%c%c%c%c",IR_Proximity[ RIGHT] & 0xFF, (IR_Proximity[ RIGHT] >> 8) & 0xFF , IR_Proximity[ REAR ] & 0xFF ,(IR_Proximity[ REAR ] >> 8) & 0xFF); 
            printf("%c%c%c%c",IR_Proximity[ GROUNDLEFT ] & 0xFF, (IR_Proximity[ GROUNDLEFT ] >> 8) & 0xFF , IR_Proximity[ GROUNDFRONTLEFT ] & 0xFF, (IR_Proximity[ GROUNDFRONTLEFT ] >> 8) & 0xFF);
            printf("%c%c%c%c",IR_Proximity[ GROUNDFRONTRIGHT ] & 0xFF, (IR_Proximity[ GROUNDFRONTRIGHT]>>8) & 0xFF  , IR_Proximity[ GROUNDRIGHT ] & 0xFF, (IR_Proximity[ GROUNDRIGHT ] >> 8) & 0xFF );
            printf("%c%c%c%c",IR_Light[ LEFT ] & 0x00FF, (IR_Light[ LEFT ]>>8) & 0x00FF  , IR_Light[ FrontLEFT ]  & 0xFF , (IR_Light[ FrontLEFT ] >> 8)  & 0xFF);
            printf("%c%c%c%c",IR_Light[ FRONT ]  & 0xFF, (IR_Light[ FRONT ] >> 8) & 0xFF , IR_Light[ FRONTRIGHT] & 0xFF, (IR_Light[ FRONTRIGHT] >> 8) & 0xFF );
            printf("%c%c%c%c",IR_Light[ RIGHT] & 0xFF, (IR_Light[ RIGHT] >> 8) & 0xFF , IR_Light[ REAR ] & 0xFF ,(IR_Light[ REAR ] >> 8) & 0xFF); 
            printf("%c%c%c%c",IR_Light[ GROUNDLEFT ] & 0xFF, (IR_Light[ GROUNDLEFT ] >> 8) & 0xFF , IR_Light[ GROUNDFRONTLEFT ] & 0xFF, (IR_Light[ GROUNDFRONTLEFT ] >> 8) & 0xFF);
            printf("%c%c%c%c",IR_Light[ GROUNDFRONTRIGHT ] & 0xFF, (IR_Light[ GROUNDFRONTRIGHT]>>8) & 0xFF  , IR_Light[ GROUNDRIGHT ] & 0xFF, (IR_Light[ GROUNDRIGHT ] >> 8) & 0xFF );
            printf("%c%c%c%c%c%c%c%c%c\r\n",PwmMotLeft, PwmMotRight, Switchs, TV_ADDR, TV_DATA, TimeTip & 0xFF, (TimeTip >> 8) &0xFF, (TimeTip >> 16) & 0xFF, (TimeTip >> 24) &0xFF );
           

}

void __C(void)
{
  unsigned char BT_name;

  SerialCounter = 0;  
  SerialCommandOK = 0;
  BT_config = 1;                                                                // Set the flag to receive more than one line
  delay_ms(500);
  printf("+++");                                                                // Deconnect the Bluetooth communication
  delay_ms(800);
  SerialCounter = 0;  
  SerialCommandOK = 0;
  printf("SET\n");                                                              // Ask for the different info of the Bluetooth module
  delay_ms(1500);
  BT_Name = SerialBuffer[54];                                                   // Save the letter which define the Type of K-Junior
  SerialCounter = 0;                                                            // Reset the serial buffer
  printf("SELECT 0\n");                                                         // Re-active the communication
  delay_ms(1000);                                                               // Wait until the communication has restart
  BT_config = 0;
  printf("c,%c\r\n",BT_name) ;                                                  // Send the return command and the type of the K-Junior (0 = Standard, P = Pedagogique)
}

//--------------------------------------------------------------------------------//
//- Write many Data on the I2C Bus                                               -//
//--------------------------------------------------------------------------------// 
#separate
void __Y(void)
{
   unsigned char Addresse = 0;
   unsigned char registre = 0;
   unsigned char rcvalue[50];
   unsigned char nb_bytes = 0;
   unsigned char i;
   Addresse = chartohex(SerialBuffer[2]);
   Addresse = (Addresse<<4) + chartohex(SerialBuffer[3]);

   registre = chartohex(SerialBuffer[5]);
   registre = (registre<<4) + chartohex(SerialBuffer[6]);
   nb_bytes = chartohex(SerialBuffer[8]);
   nb_bytes = (nb_bytes<<4) + chartohex(SerialBuffer[9]);        // Number of Data to Write

   for(i=0;i<nb_bytes;i++)
   {
      rcvalue[i] = chartohex(SerialBuffer[11+i+i+i]);
      rcvalue[i] = (rcvalue[i]<<4) + chartohex(SerialBuffer[12+i+i+i]);
      //delay_ms(1);                     // Needed for module ???
   }
   
   i2c_start();
   i2c_write( Addresse );                                           // Writing Address of the module
   i2c_write( registre );                                           // Writing Address of the register
   for(i=0;i<nb_bytes;i++)
   {
      i2c_write( rcvalue[i] );                                        // Writing Data in the register
   }


   i2c_stop();
   printf("y\r\n");
}
//--------------------------------------------------------------------------------//
//- This function will be call when a serial command is completely received      -//
//--------------------------------------------------------------------------------//  
#separate
void SerialCommandHandler (void)
{

   // If the second char is not a "," or "\r" or "\n"
   if((SerialBuffer[ 1 ] != ',') &&  (SerialBuffer[ 1 ] != 10) && (SerialBuffer[ 1 ] != 13))
   {
      SerialCounter = 0;                                                        // Flush the buffer and do nothing
      SerialCommandOK = 0;                                                      // For the Bluetooth when the connection is not made (communication with USB)
   }
   else
   {
 
      switch( SerialBuffer[ 0 ] )
      {
   //--------------------------------------------------------------------------------//
   //- Move the K-Junior during a define time in ms                                 -//
   //--------------------------------------------------------------------------------//            
         case 'A' :
           __A();
           break;
   //--------------------------------------------------------------------------------//
   //- Read the KJunior OS version and revision                                     -//
   //--------------------------------------------------------------------------------//            
         case 'B' :                                                             // KJOs Version
            __B();
            break;

         case 'C' :
            __C();
            break;
       
   //--------------------------------------------------------------------------------//
   //- Set the speed of the 2 motors. Valure are +/- 20 for each one                -//
   //--------------------------------------------------------------------------------//
         case 'D' :                                                             // Set Motors Speed
            __D();
            break;
   //--------------------------------------------------------------------------------//
   //- Read the Speed of the two motors (-20 to +20)                                -//
   //--------------------------------------------------------------------------------//               
         case 'E' :                                                             // Read Motors Speed
            __E();
            break;
   //--------------------------------------------------------------------------------//
   //- read the Time base in ms                                                     -//
   //--------------------------------------------------------------------------------//
         case 'F' :
            printf("f,%ld\r\n" TimeTip);
            break;
   //--------------------------------------------------------------------------------//
   //- reset the Time base                                                          -//
   //--------------------------------------------------------------------------------//
         case 'G' : 
            TimeTip = 0;
            printf("g\r\n");
            break;
   //--------------------------------------------------------------------------------//
   //- Set the frequency of the buzzer, 0 = Off , 1 = 20Hz, 100 = 2kHz              -//
   //--------------------------------------------------------------------------------//              
         case 'H' : 
            __H();
            break;
   //--------------------------------------------------------------------------------//
   //- read the value of the mode switch                                            -//
   //--------------------------------------------------------------------------------//                
         case 'I' :                        
            printf( "i,%x\r\n" , Switchs);
            break;
//--------------------------------------------------------------------------------//
//- Scan the I2C Bus                                                             -//
//--------------------------------------------------------------------------------//             
         case 'J' :                        // Scan I2C Bus
            __J();
            break;
         //case 'K' : Unused
   //--------------------------------------------------------------------------------//
   //- Set the LED value                                                            -//
   //--------------------------------------------------------------------------------//                
         case 'L' :   
            __L();
             break;
   //--------------------------------------------------------------------------------//
   //- Return the values of the IR Proximity sensor                                 -//
   //--------------------------------------------------------------------------------//                 
         case 'N' : 
            __N();
            break;
   //--------------------------------------------------------------------------------//
   //- Return the values of the IR Ambient light sensor                             -//
   //--------------------------------------------------------------------------------//                  
         case 'O' :                      
            __O();
            break;

 
//--------------------------------------------------------------------------------//
//- Read on the I2C Bus                                                          -//
//--------------------------------------------------------------------------------//  
         case 'R' :                        // Read I2C
            __R();
            break;

//--------------------------------------------------------------------------------//
//- Return the receiving TV data                                                 -//
//--------------------------------------------------------------------------------// 
         case 'T' :                        // Read TV Remote Sensor
            __T();
            break;

         case 'V' : 
            __V();
            break;
//--------------------------------------------------------------------------------//
//- Write a data on the I2C Bus                                                  -//
//--------------------------------------------------------------------------------//          
         case 'W' :                        // Write I2C
            __W();
            break;

//--------------------------------------------------------------------------------//
//- Read all K-Junior Data in binary mode                                        -//
//--------------------------------------------------------------------------------//
        
         case 'X' :   // Fast binary read
            __X();
            break;
   

 //--------------------------------------------------------------------------------//
//- Write many Data on the I2C Bus                                               -//
//--------------------------------------------------------------------------------//         
         case 'Y' :                                                           
            __Y();
            break;
   //--------------------------------------------------------------------------------//
   //- Reset the KJunior uC                                                         -//
   //--------------------------------------------------------------------------------//                 
         case 'Z' :
            __Z();
            break;
     
         default :   // Unknown message command
            __default();  
            break;
      }

   }
   SerialCounter = 0;
   SerialCommandOK = 0;
   

}

#INT_RDA 
void Serial_Interrupt(void)
{
 unsigned char FLUSH;
   
   while(kbhit())                                                               // Read while data available
   {
      if((SerialCommandOK == 1) && (BT_config == 0))                            // If a command is still in traitment (and no config Bluetooth is in process), Flush the buffer
      {
         FLUSH = getc();
      }
      else
      {      
         SerialBuffer[ SerialCounter ] = getc();
         if((SerialBuffer[ SerialCounter ] == 13) || (SerialBuffer[ SerialCounter ] == 10))   // '\n' & '\r'
         { 
            SerialCommandOK = 1;                                                   // Set the Received command Flag
         }
         if (SerialCounter < ( SERIAL_BUFFER_SIZE-1 ))                             // To discard buffer overflow
         {
            SerialCounter++;
         }
      }
   }
      
   
}











